/* transformFile. */
/* this is a template for simple ascii translations. eg. map CR over LF */
/* each file from inList is opened and a temp file is opened. 512 blocks are */
/* read from the file and passed to ConvertBuffers. Converted buffers are
/* written to the temp file. At the end, the temp file is exchanged for original */

#include "myFilter.h"

OSErr			DoMyError( short, short, OSErr);
char 			*pStrcpy(char *,char *); /* to copy strings for the dialogs */
short			MakeTempFile( FSSpec *, FSSpec *);
short			SaveTempToOriginal( FSSpec *, FSSpec *);
short			ConvertBuffers( char *, char *, long);	/* work done here */

char			*inBuffer, *outBuffer;
FTBlockPtr		gTheBlock;
callBackPtr		callBack;

/********************************** Main **********************/

OSErr		main( theBlock)
FTBlockPtr	theBlock;

{
	FSSpec				*inList;
	long				inCount;
	short				i, srcRefNum, dstRefNum, progress;
	long				count;
	OSErr				err;
	FSSpec				tempFSS;
	
	gTheBlock = theBlock;
	callBack = theBlock->callBackProc;
	inCount = theBlock->inFileCount;
	inList = theBlock->inFileList;
	
	inBuffer = (char *)NewPtrClear( 512);
	outBuffer = (char *)NewPtrClear( 512);

	theBlock->progress = 0;			/* pass 0 to start progress window */
	theBlock->message = DO_PROGRESS;
	(*callBack)( theBlock);			/* Display the Progress Window */
	
	for( i = 0; i < inCount; ++i)			/* index thru the files */
	{
		
		theBlock->progress = (i + 1) * (100 / inCount);
		pStrcpy((char *)theBlock->string0, (char *)"\p Make Lower Case");	/* Filter Name */
		pStrcpy((char *)theBlock->string1, (char *)inList[i].name);		/* Current File */
		theBlock->message = DO_PROGRESS;
		(*callBack)( theBlock);			/* Update the Progress Window */

		err = MakeTempFile( &inList[i], &tempFSS);
		if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);
		err = FSpOpenDF( &tempFSS, fsCurPerm, &dstRefNum);
		if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);
		err = FSpOpenDF( &inList[i], fsCurPerm, &srcRefNum);
		if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);

		count = 512;		
		while ( ( err = FSRead( srcRefNum, &count, inBuffer) ) == noErr )
		{
			ConvertBuffers( inBuffer, outBuffer, count);				
			err = FSWrite( dstRefNum, &count, outBuffer);
			if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);
			
			theBlock->message = FILTER_HANDLE_EVENT;
			(*callBack)( theBlock);		/* if using the Progress window, make sure user didn't hit Stop */
			if( theBlock->message == -1) return DoMyError( srcRefNum, dstRefNum, -1);	/* user has hit the stop button in the Progress window */
		}
		if( err == eofErr) /* grab the last little bit */
		{
			ConvertBuffers( inBuffer, outBuffer, count);
			err = FSWrite( dstRefNum, &count, outBuffer);
			if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);
		}
		else if( err != noErr) return DoMyError( srcRefNum, dstRefNum, err);
		FSClose( srcRefNum);
		FSClose( dstRefNum);
		err = SaveTempToOriginal( &inList[i], &tempFSS);
		if( err != noErr)  return DoMyError( srcRefNum, dstRefNum, err);
	}

	theBlock->progress = 101;
	theBlock->message = DO_PROGRESS;
	(*callBack)( theBlock);			/* Dispose the Progress Window */

	DisposPtr( (Ptr)inBuffer );
	DisposPtr( (Ptr)outBuffer );
	return err;
}

/************************************ DoMyError **************************/

OSErr	DoMyError( srcRefNum, dstRefNum, err)
short	srcRefNum, dstRefNum;
OSErr	err;
{
	gTheBlock->progress = 101;			/* pass any number above 100 to dispose */
	gTheBlock->message = DO_PROGRESS;
	(*callBack)( gTheBlock);			/* Dispose the Progress Window */
	
	FSClose( srcRefNum);
	FSClose( dstRefNum);
	DisposPtr((Ptr)inBuffer);
	DisposPtr((Ptr)outBuffer);
	return err;
}

/********************************** pStrcpy *************************************/

char *pStrcpy(s,t)
	char *s, *t;
{
	short	tLen;

	for (tLen = *t + 1; tLen--; s[tLen] = t[tLen]);
	return (s);
}

/****************************** MakeTempFile ****************************/

short			MakeTempFile( myFSS, tempFSS)
FSSpec			*myFSS, *tempFSS;
{
	OSErr		err;
	long		secs, tempDirID;
	Str255		tempFileName;
	short		tempVRefNum;
	
	secs = Time;
	NumToString( secs, tempFileName);
	tempDirID = myFSS->parID;
	tempVRefNum = myFSS->vRefNum;
	
	err = FSMakeFSSpec( tempVRefNum, tempDirID, tempFileName, tempFSS);
	if( err != noErr && err != -43) return err;
	err = FSpCreate( tempFSS, 'trsh', 'trsh', -1);
	if( err != noErr) return err;
	return noErr;
}

/****************************** SaveTempToOriginal ****************************/

short			SaveTempToOriginal( myFSS, tempFSS)
FSSpec			*myFSS, *tempFSS;
{
	OSErr		err;

	err = FSpExchangeFiles( tempFSS, myFSS);
	if( err != noErr) return err;
	err = FSpDelete( tempFSS);
	if( err != noErr) return err;
	return noErr;
}	

/****************************** ConvertBuffers ****************************/
/* this is where the real work is done */

short		ConvertBuffers( inBuf, outBuf, count)
char		*inBuf, *outBuf;	
long		count;
{
	short	i;
	
	for( i = 0; i < count; ++i)
	{
		if( inBuf[i] > 64 && inBuf[i] < 91 )
			outBuf[i] = inBuf[i] + 32;
		else
			outBuf[i] = inBuf[i];
	}
	return noErr;
}			
