#ifndef	_myFilter_
#define	_myFilter_

#define				FILTER_HANDLE_EVENT		1	/* to allow app to handle an event */
#define				DO_ALERT_STRING			2	/* to get app to display alert string(s) */
#define				DO_PROGRESS				3	/* filter progress ( 1 - 100 ) */
#define				GET_USER_STRING			4	/* get app to obtain string from user */

typedef struct
{
	long		inFileCount, *outFileCount;			/* file counts for FSSpec pointer tables */
	FSSpec		*inFileList, *outFileList;			/* pointer tables to FSSpecs */
	StringPtr	string0, string1, string2, string3;	/* strings for dialogs */
	short		message, progress;					/* callback message */
	ProcPtr		callBackProc;						/* pointer to callBack routine */
} FTBlock, *FTBlockPtr;

typedef short 	(*callBackPtr)( FTBlockPtr);   /* ptr to function returning OSErr */

#endif


/*

------------Defines-----------------

The defined messages represent all the current services obtainable from the app.
Set the message field to the appropriate defined message and call the callback
proc to order the required service.

FILTER_HANDLE_EVENT tell the app to handle an event. Pass often if filter long.

DO_ALERT_STRING Get app to display an alert. Use the 4 strings in the FTBlock to
fill out the text in the alert.

DO_PROGRESS	Set from 1 to 100 to indicate progress of filter.

GET_USER_STRING	Get a string from the user.

------------FTBlock Structure-------------------

inFileCount, *outFileCount

inFileCount tells the filter how many files are coming.
*outFileCount can be used by the filter to tell the app (or next filter) 
how many files are leaving. Note. If the *outFileCount is greater than
0 then the next filter in line will get the current filters output files
as its input files.

*inFileList, *outFileList
*inFileList gives the filter an array of FSSpecs for incoming files.
*outFileList can be used by the filter to give the app (or next filter)
an array of FSSpecs for outgoing files.
Note. If you transform the original files without creating any new files,
then there is no need to fill out *outFileCount and *outFileList.

string0, string1, string2, string3
Can be filled out by the filter and (with a DO_ALERT_STRING message) given
to the app for display in an alert dialog.

message, progress
message allows messages from the filter to the app and vice versa. Use the
defined messages to fill out the message field. Check the message field
regularly if using Progress Window for a -1. 

progress
Can be given a value from 1 to 100 and (with a DO_PROGRESS message) given
to the app for the progress window. 
Note. a value of 0 will display the progress window, a value greater than
100 will hide the progress window. 
Note. If using the progress window, the filter should regularly check the
message field for a -1 (user cancel)

callBackProc
Pointer to the callback proc in the app. Dereference it and call it with
an appropriate message. See some of the sample code for examples.

*/