/* appleEvents.c */

#include			"myAEevents.h"
#include			"fileUtils.h"
#include			"myString.h"
#include			"windowUtils.h"

/************* Globals *********/

extern Boolean		gDone;
extern ListHandle	gFileList, gFilterList;

/****************************** CheckAEAvailable **************************/

Boolean		CheckAEAvailable( void)
{
	OSErr	gestaltErr;
	long	gestaltAnswer;
	
	gestaltErr = Gestalt( gestaltAppleEventsAttr, &gestaltAnswer);
	if( gestaltErr)
		return FALSE;
	return TRUE;
}

/*************************** InitAppleEvents **********************************/

OSErr 		InitAppleEvents ()
{
	OSErr	err;

	err = noErr;
	if ( err == noErr) err = AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
			(ProcPtr)HandleOApp, 0, FALSE);
	if ( err == noErr) err = AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments, 
			(ProcPtr)HandleODocs, 0, FALSE);
	if ( err == noErr) err = AEInstallEventHandler(kCoreEventClass, kAEPrintDocuments,
			(ProcPtr)HandlePrint, 0, FALSE);
	if (err == noErr) err = AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
			(ProcPtr)HandleQuit, 0, FALSE);
	return( err);
}

/************************* HandleOAPP *************************/

pascal OSErr	HandleOApp( AppEvIn, AppEvOut, Refcon)
AppleEvent		*AppEvIn, *AppEvOut;
long			Refcon;
{
	OSErr		err;
	
	err = MyGotRequiredParams( AppEvIn);
	return err;
}

/************************* HandleODocs *************************/

pascal OSErr	HandleODocs( AppEvIn, AppEvOut, Refcon)
AppleEvent		*AppEvIn, *AppEvOut;
long			Refcon;
{
	FSSpec		myFSS;
	AEDescList	docList;
	OSErr		err;
	long		index, itemsInList;
	Size		actualSize;
	AEKeyword	keywd;
	DescType	returnedType;
	
	err = AEGetParamDesc( AppEvIn, keyDirectObject, typeAEList, &docList);
	if( err != noErr) 
		DoError( err);
	err = MyGotRequiredParams( AppEvIn);
	if( err != noErr)
	{
		DoError( err);
		AEDisposeDesc( &docList);
		return err;
	}
	err = AECountItems( &docList, &itemsInList);
	
	LDoDraw( FALSE, gFileList );				/* turn lists drawing off */
	LDoDraw( FALSE, gFilterList );		
			
	for( index = 1; index <= itemsInList; ++index)
	{
		err = AEGetNthPtr( &docList, index, typeFSS, &keywd, 
				&returnedType, (Ptr)&myFSS, sizeof( myFSS), &actualSize);
		if( err != noErr) 
			DoError( err);
		else
		{
			err = MyOpenFile(&myFSS);
			if( err != noErr) 
				DoError( err);
		}
	}
	DrawList( gFileList);					/* turn on lists and draw them */
	DrawList( gFilterList);
	
	err = AEDisposeDesc( &docList);
	return noErr;
}

/************************* HandlePrint *************************/

pascal OSErr	HandlePrint( AppEvIn, AppEvOut, Refcon)
AppleEvent		*AppEvIn, *AppEvOut;
long			Refcon;
{
	return noErr;
}

/************************* HandleQUIT *************************/

pascal OSErr	HandleQuit( AppEvIn, AppEvOut, Refcon)
AppleEvent		*AppEvIn, *AppEvOut;
long			Refcon;
{
	Boolean		userCancel;
	OSErr		err;
	
	err = MyGotRequiredParams( AppEvIn);
	if( err != noErr)
		return err;
	userCancel = MyPrepareToTerminate();
	if ( userCancel)
		return userCancel;
	else
		return noErr;
}

/************************* MyGotRequiredParams ********************/

OSErr			MyGotRequiredParams( AppEvent)
AppleEvent		*AppEvent;
{
	DescType	returnedType;
	Size		actualSize;
	OSErr		err;
	
	err = AEGetAttributePtr( AppEvent, keyMissedKeywordAttr, 
			typeWildCard, &returnedType, 0L, 0, &actualSize);
	if( err == errAEDescNotFound)
		return noErr;
	else if( err == noErr)
		return errAEEventNotHandled;
	else
		return err;
}

/************************ MyPrepareToTerminate ***************************/

Boolean		MyPrepareToTerminate( void)
{
	gDone = TRUE;
	return FALSE;
}

/********************** ErrorStringHandler ******************************/

void	ErrorStringHandler( ErrorString )
Str255		ErrorString;
{
	ParamText( ErrorString, "\p", "\p", "\p" );
	StopAlert( 400, 0L);
}

/******************************* DoError ******************************/

void			DoError( err)
OSErr			err;
{
	Str255		errorString;
	
	switch (err)
	{
	case noErr:
		pStrcpy((char *)errorString, (char *)"\pnoErr");
		ErrorStringHandler( errorString );
		break;
	case paramErr:
		pStrcpy((char *)errorString, (char *)"\pparamErr");
		ErrorStringHandler( errorString );
		break;
	case memFullErr:
		pStrcpy((char *)errorString, (char *)"\pmemFullErr");
		ErrorStringHandler( errorString );
		break;
	case bufferIsSmall:
		pStrcpy((char *)errorString, (char *)"\pbufferIsSmall");
		ErrorStringHandler( errorString );
		break;
	case noOutstandingHLE:
		pStrcpy((char *)errorString, (char *)"\pnoOutstandingHLE");
		ErrorStringHandler( errorString );
		break;
	case connectionInvalid:
		pStrcpy((char *)errorString, (char *)"\pconnectionInvalid");
		ErrorStringHandler( errorString );
		break;
	case errAECoercionFail:
		pStrcpy((char *)errorString, (char *)"\perrAECoercionFail");
		ErrorStringHandler( errorString );
		break;
	case errAEDescNotFound:
		pStrcpy((char *)errorString, (char *)"\perrAEDescNotFound");
		ErrorStringHandler( errorString );
		break;
	case errAECorruptData:
		pStrcpy((char *)errorString, (char *)"\perrAECorruptData");
		ErrorStringHandler( errorString );
		break;
	case errAEWrongDataType:
		pStrcpy((char *)errorString, (char *)"\perrAEWrongDataType");
		ErrorStringHandler( errorString );
		break;
	case errAENotAEDesc:
		pStrcpy((char *)errorString, (char *)"\perrAENotAEDesc");
		ErrorStringHandler( errorString );
		break;
	case errAEBadListItem:
		pStrcpy((char *)errorString, (char *)"\perrAEBadListItem");
		ErrorStringHandler( errorString );
		break;
	case errAENewerVersion:
		pStrcpy((char *)errorString, (char *)"\perrAENewerVersion");
		ErrorStringHandler( errorString );
		break;
	case errAENotAppleEvent:
		pStrcpy((char *)errorString, (char *)"\perrAENotAppleEvent");
		ErrorStringHandler( errorString );
		break;
	case errAEEventNotHandled:
		pStrcpy((char *)errorString, (char *)"\perrAEEventNotHandled");
		ErrorStringHandler( errorString );
		break;
	case errAEReplyNotValid:
		pStrcpy((char *)errorString, (char *)"\perrAEReplyNotValid");
		ErrorStringHandler( errorString );
		break;
	case errAEUnknownSendMode:
		pStrcpy((char *)errorString, (char *)"\perrAEUnknownSendMode");
		ErrorStringHandler( errorString );
		break;
	case errAEWaitCanceled:
		pStrcpy((char *)errorString, (char *)"\perrAEWaitCanceled");
		ErrorStringHandler( errorString );
		break;
	case errAETimeout:
		pStrcpy((char *)errorString, (char *)"\perrAETimeout");
		ErrorStringHandler( errorString );
		break;
	case errAENoUserInteraction:
		pStrcpy((char *)errorString, (char *)"\perrAENoUserInteraction");
		ErrorStringHandler( errorString );
		break;
	case errAENotASpecialFunction:
		pStrcpy((char *)errorString, (char *)"\perrAENotASpecialFunction");
		ErrorStringHandler( errorString );
		break;
	case errAEParamMissed:
		pStrcpy((char *)errorString, (char *)"\perrAEParamMissed");
		ErrorStringHandler( errorString );
		break;
	case errAEUnknownAddressType:
		pStrcpy((char *)errorString, (char *)"\perrAEUnknownAddressType");
		ErrorStringHandler( errorString );
		break;
	case errAEHandlerNotFound:
		pStrcpy((char *)errorString, (char *)"\perrAEHandlerNotFound");
		ErrorStringHandler( errorString );
		break;
	case errAEReplyNotArrived:
		pStrcpy((char *)errorString, (char *)"\perrAEReplyNotArrived");
		ErrorStringHandler( errorString );
		break;
	case errAEIllegalIndex:
		pStrcpy((char *)errorString, (char *)"\perrAEIllegalIndex");
		ErrorStringHandler( errorString );
		break;
	default:
		break;
	}
}

