#include	"events.h"
#include	"windowUtils.h"
#include	"cursors.h"
#include	"filter.h"
#include	"fileUtils.h"

/**************************** globals *********************/

extern FSSpec			*gInFileArray, *gOutFileArray;					/* FSSpec pointer tables */
extern WindowPtr		gFilesWindow, gFiltersWindow, gProgressWindow;
extern ListHandle		gFileList, gFilterList, gPipeList;
extern ControlHandle	gDeleteButton, gDeleteFiltersButton, gPipesButton, gProgressStopButton;
extern Rect				gFilterListRect, gFileListRect, gPipeListRect;
extern Boolean			gStopProcessing;
Boolean					gDone, gWNEImplemented, gInBackground;
extern FTBlockPtr		gTheFTBlockPtr;									/* pointer to current FTBlock */

/************************ MainLoop ********************************/

void		MainLoop( void )
{
	OSErr	err;
	
	gDone = FALSE;
	gInBackground = FALSE;
	gWNEImplemented = (NGetTrapAddress(0x60, ToolTrap) !=
						NGetTrapAddress(0x9F, ToolTrap));
	
	while (!gDone)
		HandleEvent();
}

/************************* HandleEvent ***************************/

void				HandleEvent( void )
{
	Boolean			doIt;
	EventRecord		event;
	
	if (gWNEImplemented)
		doIt = WaitNextEvent( everyEvent, &event, 0L, 0L);
	else
	{
		SystemTask();
		doIt = GetNextEvent( everyEvent, &event);
	}

	if( doIt == TRUE || event.what == nullEvent) 
	{
		switch (event.what)
		{
			case updateEvt:
				HandleUpdate( (WindowPtr)event.message );
				break;

			case activateEvt:
				HandleActivate((WindowPtr)event.message, 
						(event.modifiers & activeFlag) != 0 );
				break;

			case app4Evt:
				if ( ( event.message & 0x0001 ) == 1	/* Resuming */
				|| ( event.message & 0x0001 ) == 0 )	/* Suspending */
				{
					gInBackground = (event.message & 0x01 ) == 0;
					HandleActivate(FrontWindow(), !gInBackground);
				}
				break;

			case kHighLevelEvent:
				HandleHighLevelEvent( &event);
				break;

			case mouseDown:
				HandleMouseDown( &event);
				break;

	 		case keyDown:
			case autoKey:
				if (( event.modifiers & cmdKey ) !=0)
					HandleMenuChoice( MenuKey( event.message));

			case nullEvent:
				break;

			default:
				break;
		}
	}
}

/*************************** HandleActivate ***********************/

void		HandleActivate(window, becomingActive)
WindowPtr	window;
Boolean		becomingActive;
{
	 GrafPtr	oldPort;
	 
	 GetPort(&oldPort);
	if (window == gFilesWindow)
	{		
		DrawGrowIcon( gFilesWindow );
		
		if ( becomingActive )
		{
			SetPort( gFilesWindow );
			LActivate(true, gFileList);
		}
		else
			LActivate(false, gFileList);
	}
	if (window == gFiltersWindow)
	{		
		DrawGrowIcon( gFiltersWindow );
		
		if ( becomingActive )
		{
			SetPort( gFiltersWindow );
			LActivate(true, gFilterList);
			LActivate(true, gPipeList);
		}
		else
		{
			LActivate(false, gFilterList);
			LActivate(false, gPipeList);
		}
	}
	if (window == gProgressWindow)
	{		
		if ( becomingActive )
			SetPort( gProgressWindow );
	}
}
		
/*************************** HandleUpdate ***********************/

void		HandleUpdate(window)
WindowPtr	window;
{
	GrafPtr		oldPort;
	Rect		r;

	GetPort( &oldPort );
	SetPort( window );
	
	BeginUpdate( window );

	if (window == gFilesWindow)
	{
		DrawGrowIcon( gFilesWindow );
		DrawControls( gFilesWindow );
		TextFont( 0 );
		TextSize( 12);
		LUpdate(gFilesWindow->visRgn, gFileList);
	}
	else if (window == gFiltersWindow)
	{
		DrawControls( gFiltersWindow );
		DrawGrowIcon( gFiltersWindow );
		TextFont( 0 );
		TextSize( 12);
		LUpdate(gFiltersWindow->visRgn, gFilterList);
		LUpdate(gFiltersWindow->visRgn, gPipeList);
	}
	else if (window == gProgressWindow)
	{
		DrawControls( gProgressWindow);
		TextFont(0);
		TextSize( 12);
		r.top = gProgressWindow->portRect.top;
		r.bottom = r.top + 50;
		r.left = 99;
		r.right = gProgressWindow->portRect.right;
		EraseRect(&r);
		MoveTo( 15, 20);
		DrawString( "\pFilter Name: ");
		MoveTo( 100, 20);
		if( gTheFTBlockPtr != 0L) DrawString( gTheFTBlockPtr->string0);
		MoveTo( 15, 45);
		DrawString( "\pFile Name: ");
		MoveTo( 100, 45);
		if( gTheFTBlockPtr != 0L) DrawString( gTheFTBlockPtr->string1);
		r.bottom = gProgressWindow->portRect.bottom - 12;
		r.top = r.bottom - 14;
		r.left = 15;
		r.right = r.left + 202;
		FrameRect( &r);
	}
	EndUpdate( window);
	SetPort( oldPort);
}

/************************ HandleMouseDown ***************************/

void				HandleMouseDown( event)
EventRecord			*event;
{
	WindowPtr		whichWindow;
	short			thePart, tempGrowHeight;
	long			menuChoice;
	long			windSize;
	GrafPtr			oldPort;
	WindowPeek		wp;
	WStateData		*p;
	Rect			maxWindowRect;
				
	thePart = FindWindow(event->where, &whichWindow);
	
	switch (thePart)
	{
		case inMenuBar:
			menuChoice = MenuSelect(event->where);
			HandleMenuChoice( menuChoice);
			break;

		case inSysWindow :
			SystemClick( event, whichWindow);
			break;

		case inDrag :
			DragWindow( whichWindow, event->where, &(screenBits.bounds));
			break;

		case inContent :
			if ( whichWindow != FrontWindow())
				SelectWindow( whichWindow);
			else
				HandleContentClick( whichWindow, event);
			break;

		case inGoAway :
			if ( TrackGoAway( whichWindow, event->where))
			{
				HideWindow(whichWindow);
				if( whichWindow != gProgressWindow)
					gDone = TRUE;
			}
			break;

		case inGrow :
		 	maxWindowRect = screenBits.bounds;	/* set up limits so we only grow vertically */
		 	maxWindowRect.top = 99;
		 	if( whichWindow == gFiltersWindow)
		 	{
			 	maxWindowRect.left = whichWindow->portRect.right +1;
			 	maxWindowRect.right = whichWindow->portRect.right +1;
			 }
			 else if( whichWindow == gFilesWindow)
		 	{
			 	maxWindowRect.left = whichWindow->portRect.right +1;
			 	maxWindowRect.right = whichWindow->portRect.right +1;
			 }
			windSize = GrowWindow( whichWindow, event->where, &maxWindowRect);	
			if ( windSize != 0)
			{
				EraseRect( &whichWindow->portRect);
				tempGrowHeight = HiWord( windSize);
				tempGrowHeight -= 15;
				tempGrowHeight = (tempGrowHeight / 18) * 18;
				tempGrowHeight += 15;
				SizeWindow( whichWindow, LoWord( windSize), 
							tempGrowHeight, TRUE);
				if( whichWindow == gFilesWindow)
				{
					AdjustWindowItems( whichWindow);
					SetUpDeleteButton ( whichWindow);
				}
				else if( whichWindow == gFiltersWindow)
				{
					AdjustWindowItems( whichWindow);
					SetUpFilterButtons ( whichWindow);
				}
			}
			break;

		case inZoomIn :
		case inZoomOut :
			if ( TrackBox( whichWindow, event->where, thePart))
			{
				EraseRect( &whichWindow->portRect);
				if( whichWindow == gFilesWindow || whichWindow == gFiltersWindow)
				{
					wp = (WindowPeek)whichWindow;	/* set up limits so we only grow vertically */
					p = * (WStateData **) wp->dataHandle;
				 	(*p).stdState.left = (*p).userState.left;
				 	(*p).stdState.right = (*p).userState.right;
				 }
				ZoomWindow( whichWindow, thePart, FALSE);
				if( whichWindow == gFilesWindow)
				{
					AdjustWindowItems( whichWindow);
					SetUpDeleteButton ( whichWindow);
				}
				else
				if( whichWindow == gFiltersWindow)
				{
					AdjustWindowItems( whichWindow);
					SetUpFilterButtons ( whichWindow);
				}
			}
			break;
	}
}

/***************************** HandleContentClick **************************/

void				HandleContentClick( window, event)
WindowPtr			window;
EventRecord			*event;
{
	short			theLine, thePart;
	Point			where, mouse, lastClick, cell;
	ControlHandle	control;
	Rect			r;
	
	mouse = event->where;
	where = mouse;
	GlobalToLocal(&where);
		
	if ( window == gFilesWindow)
	{
		LClick(where, event->modifiers, gFileList);
		HandleListDrag( gFileList);

		if ( ( thePart = FindControl( where, gFilesWindow, &control)) != 0)
		{
			if ( control == gDeleteButton) 
			{
				if ( TrackControl( gDeleteButton, where, 0L))
				{
					SetPt( &cell, 0, 0);
					if( LGetSelect( TRUE, &cell, gFileList))
					{
						while( LGetSelect( TRUE, &cell, gFileList))
							LDelRow( 1, cell.v, gFileList);
					}	
					else if ((**gFileList).dataBounds.bottom != 0)						/* set files list to 0 */
						LDelRow( (**gFileList).dataBounds.bottom, 0, gFileList);
				}
			}
		}
	}
	if ( window == gFiltersWindow)
	{
		r = ( **gFilterList).rView;
		r.right += 15;
		if( PtInRect( where, &r))
		{
			if ( LClick(where, event->modifiers, gFilterList))
			{
				lastClick = LLastClick( gFilterList);
				theLine = lastClick.v;
				/*HandleFilterLineSelect(theLine);*/
				HandleSingleFilterSelect( theLine);	
			}
			HandleFilterListDrag( gFilterList);
		}
		else
		{
			if ( LClick(where, event->modifiers, gPipeList))
			{
				lastClick = LLastClick( gPipeList);
				theLine = lastClick.v;
				HandlePipeLineSelect(theLine);
			}
			HandleListDrag( gPipeList);
		}
		if ( ( thePart = FindControl( where, gFiltersWindow, &control)) != 0)
		{
			if ( control == gDeleteFiltersButton) 
			{
				if ( TrackControl( gDeleteFiltersButton, where, 0L))
				{
					SetPt( &cell, 0, 0);
					if( LGetSelect( TRUE, &cell, gPipeList))
					{
						while( LGetSelect( TRUE, &cell, gPipeList))
							LDelRow( 1, cell.v, gPipeList);
					}	
					else if ((**gPipeList).dataBounds.bottom != 0)						/* set FilterPipe list to 0 */
						LDelRow( (**gPipeList).dataBounds.bottom, 0, gPipeList);
				}
			}
			if ( control == gPipesButton) 
			{
				if ( TrackControl( gPipesButton, where, 0L)) 
					PipeFilters();
			}
		}
	}
	if ( window == gProgressWindow)
	{
		if ( ( thePart = FindControl( where, gProgressWindow, &control)) != 0)
		{
			if ( control == gProgressStopButton) 
			{
				if ( TrackControl( gProgressStopButton, where, 0L))
				{
					HideWindow( gProgressWindow);
					gStopProcessing = TRUE;
				}
			}
		}
	}
}

/***************************** HandleFilterLineSelect *********************/

void		HandleFilterLineSelect( theLine)
short		theLine;
{
	short		pipeListBottom, i, length = 63, pipeInfoSize = sizeof(PipeInfo), shortSize = sizeof( short);
	Point		cell, hiddenCell;
	char		line[63];
	PipeInfo	myPipeInfo;
	Rect		cellRect;
	
	myPipeInfo.filterLine = theLine;
	SetPt( &cell, 0, theLine);						/* insert line in list */
	LGetCell( &line[1], &length, cell, gFilterList);
	line[0] = length;
	LDoDraw( FALSE, gPipeList);
	AddLineToList( gPipeList, (StringPtr)line);
	cell.h = 1;
	LGetCell( &myPipeInfo.filterType, &shortSize, cell, gFilterList);
	SetPt( &hiddenCell, 1, ( **gPipeList).dataBounds.bottom -1);			
	LSetCell( &myPipeInfo, sizeof(myPipeInfo), hiddenCell, gPipeList);		/* insert short in new cell */
	hiddenCell.h = 0;
	LDoDraw( TRUE, gPipeList);
	LDraw( hiddenCell, gPipeList);
}

/***************************** HandlePipeLineSelect *********************/

void		HandlePipeLineSelect( theLine)
short		theLine;
{
	long		inFileCount, outFileCount;
	Point		cell;
	short		filtNum, pipeInfoSize = sizeof(PipeInfo);
	PipeInfo	myPipeInfo;
	
	inFileCount = ( **gFileList).dataBounds.bottom;
	outFileCount = 0;
	if( inFileCount > 0)
	{
		WatchCursor();
		SetPt( &cell, 1, theLine);
		LGetCell( &myPipeInfo, &pipeInfoSize, cell, gPipeList);
		filtNum = myPipeInfo.filterLine;	
		SetupFileArray();	
		FilterFiles( filtNum, inFileCount, gInFileArray, 
					 &outFileCount, gOutFileArray);
		ArrowCursor();
	}
}

/***************************** HandleSingleFilterSelect *********************/

void		HandleSingleFilterSelect( theLine)
short		theLine;
{
	long		inFileCount, outFileCount;
	
	inFileCount = ( **gFileList).dataBounds.bottom;
	outFileCount = 0;
	if( inFileCount > 0)
	{
		WatchCursor();
		SetupFileArray();	
		FilterFiles( theLine, inFileCount, gInFileArray, 
					 &outFileCount, gOutFileArray);
		ArrowCursor();
	}
}

/***************************** PipeFilters *********************/

void		PipeFilters( void)
{
	long	inFileCount, outFileCount;
	short	i, j, filtNum, pipeInfoSize;
	FSSpec	*tempFileArray;
	Cell	thePipeCell;
	OSErr	err;
	PipeInfo	myPipeInfo;
	
	inFileCount = ( **gFileList).dataBounds.bottom;
	outFileCount = 0;
	if(inFileCount > 0)
	{
		for( i = 0; i < ( **gPipeList).dataBounds.bottom; ++i)	/* deselect all items */
		{
			SetPt( &thePipeCell, 0, i);
			LSetSelect( FALSE, thePipeCell, gPipeList);
		}
		DrawList( gPipeList);
		SetupFileArray();	
		for( i = 0; i < ( **gPipeList).dataBounds.bottom; ++i)	/* step thru each pipe */
		{
			WatchCursor();
			SetPt( &thePipeCell, 0, i);
			LSetSelect( TRUE, thePipeCell, gPipeList);	
			LAutoScroll( gPipeList);
			SetPt( &thePipeCell, 1, i);
			pipeInfoSize = sizeof(PipeInfo);
			LGetCell( &myPipeInfo, &pipeInfoSize, thePipeCell, gPipeList);
			filtNum = myPipeInfo.filterLine;	
			if( err = FilterFiles( filtNum, inFileCount, gInFileArray, &outFileCount, gOutFileArray) != noErr) break;
			if( outFileCount > 0)
			{
				tempFileArray = gInFileArray;
				gInFileArray = gOutFileArray;
				gOutFileArray = tempFileArray;
				inFileCount = outFileCount;
				outFileCount = 0;
				if ( ( **gFileList).dataBounds.bottom != 0)						/* set File list to 0 */
					LDelRow( ( **gFileList).dataBounds.bottom, 0, gFileList);
					
				/*for( j = 0; j < inFileCount; ++j)		*/	/* add the "new" files to the File list */
						/*MyOpenFile( &gInFileArray[j]);*/	/* maybe better not to do this. Make the user put new files in */
			}
		}
	}
	ArrowCursor();
}

/********************************* HandleHighLevelEvent *******************/

void			HandleHighLevelEvent( event)
EventRecord		*event;
{
	AEProcessAppleEvent( event);
}

/*************************************** DoOpenChoice ************************/

void					DoOpenChoice()
{
    StandardFileReply 	myReply;

	StandardGetFile(nil, -1, nil, &myReply);  /* grab file */
	if (!myReply.sfGood) return;
	MyOpenFile( &myReply.sfFile);
}