/* filter.c */

#include	"filter.h"
#include	"fileUtils.h"
#include	"myString.h"
#include	"windowUtils.h"
#include	"events.h"
#include	"myDialogs.h"

/******** Defines */

#define	FilterType		'FILT'
#define	Creator			'FLTP'
#define	FilterID		0

/************* Globals */

static long				appDir;
funcHdl					*filters;			/* funcHdl pointer table */
extern WindowPtr		gProgressWindow;
Boolean					gStopProcessing;
extern ControlHandle	gProgressStopButton;
FTBlockPtr				gTheFTBlockPtr;

/************************** GetAppDir **************************/
/* get App's directory for finding the filters folder */

void			GetAppDir() 
{
	HFileInfo	myCPB;
	OSErr		myErr = noErr;
	
	myCPB.ioNamePtr = (StringPtr) NewPtrClear(40L);					
	myCPB.ioVRefNum = 0;
	myCPB.ioFRefNum = 0;
	myCPB.ioFDirIndex = -1;
	myCPB.ioNamePtr[0] = 0;
	myCPB.ioDirID = 0;
	myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
	appDir = myCPB.ioDirID;
	DisposPtr((Ptr) myCPB.ioNamePtr);
}

/************************** AddFilters **************************/
/* add the plug-in filters to the filter List */

void			AddFilters( listH)
ListHandle		listH;
{
	long		dirID;
	short		index, fRef, resNumber, i;
	HFileInfo	myCPB;
	OSErr		myErr = noErr, resErr = noErr;
	Point		cell;
	FSSpec		newFSS;
	
	dirID = appDir;
	myCPB.ioNamePtr = (StringPtr) NewPtrClear(40L);
	index = 1;
	resNumber = 0;
	while( myErr == noErr) 
	{
		myCPB.ioVRefNum = 0;
		myCPB.ioFDirIndex = index;
		myCPB.ioNamePtr[0] = 0;
		myCPB.ioDirID = dirID;
		myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
		if (myErr == noErr) 
		{
			if (((myCPB.ioFlAttrib >> 4) & 0x01) == 1)	/* directory */
			{
				if( PStrCompare((char*)myCPB.ioNamePtr, (char*)"\pFilters") == TRUE) 
				{
					index = 1;
					dirID = myCPB.ioDirID;
					LDoDraw( FALSE, listH );				/* turn file list drawing off */
					while( myErr == noErr) 
					{
						myCPB.ioVRefNum = 0;
						myCPB.ioFDirIndex = index;
						myCPB.ioNamePtr[0] = 0;
						myCPB.ioDirID = dirID;
						myErr = PBGetCatInfo((CInfoPBPtr) &myCPB, FALSE);
						if( myErr == noErr) 
						{
							if( myCPB.ioFlFndrInfo.fdCreator == Creator )
							{
								if( myCPB.ioFlFndrInfo.fdType == 'FIL1'
								|| myCPB.ioFlFndrInfo.fdType == 'FIL2'
								|| myCPB.ioFlFndrInfo.fdType == 'FIL3'
								|| myCPB.ioFlFndrInfo.fdType == 'FIL4')
								{
									FSMakeFSSpec( 0L, dirID, myCPB.ioNamePtr, &newFSS);
									fRef = FSpOpenResFile( &newFSS, fsCurPerm);
									resErr = ResError();
									if( resErr == noErr && fRef != -1 && resNumber < MAXFILTERS) 
									{
	/* Here's the action --> */			filters[resNumber] = (funcHdl)GetResource(  myCPB.ioFlFndrInfo.fdType, FilterID);
										if( filters[resNumber] != 0L) 
										{
											DetachResource( (Handle)filters[resNumber]);
											AddLineToList( listH, myCPB.ioNamePtr);
											AddFilterIcon( &newFSS);
											resNumber += 1;
										}
										else resErr = resNotFound;
										CloseResFile( fRef);
									}
								}
							}
						}
						index += 1;
					}
					DrawList( listH);
				}
			}
		}
		index += 1;
	}
	DisposPtr((Ptr) myCPB.ioNamePtr);
}

/********************************** AddSingleFilter *************************/

void		AddSingleFilter( list, myFSS)
ListHandle	list;
FSSpec		*myFSS;
{
	short	fRef, resNumber;
	OSErr	resErr;
	
	resNumber = (**list).dataBounds.bottom;
	if( resNumber < 0) resNumber = 0;
	
	fRef = FSpOpenResFile( myFSS, fsCurPerm);
	resErr = ResError();
	if( resErr == noErr && fRef != -1 && resNumber < MAXFILTERS) 
	{
		filters[resNumber] = (funcHdl)GetResource( FilterType, FilterID);
		if( filters[resNumber] != 0L) 
		{
			DetachResource( (Handle)filters[resNumber]);
			MoveHHi( (Handle)filters[ resNumber ]);
			HLock( (Handle)filters[ resNumber ]);
		}
	}
	CloseResFile( fRef);
}
	



/****************************** lockFilterHandles *******************************/

void		lockFilterHandles()
{
	short	i;
	
	for ( i = 0; i < MAXFILTERS; ++i )
	{
		if( filters[ i ] != 0L )
		{
			MoveHHi( (Handle)filters[ i ]);
			HLock( (Handle)filters[ i ]);
		}
	}
}

/****************************** unLockFilterHandles *******************************/

void		unLockFilterHandles()
{
	short	i;
	
	for ( i = 0; i < MAXFILTERS; ++i )
	{
		if( filters[ i ] != 0L )
		{
			HUnlock( (Handle)filters[ i ]);
			DisposHandle( (Handle)filters[ i ]);
		}
	}
}

/**************************** FilterFiles ************************/
/* create a FTBlock, fill it out, call the filter with a pointer to the block */

OSErr			FilterFiles( filterNumber, inFileCount, inFileArray, outFileCount, outFileArray)
short			filterNumber;
long			inFileCount, *outFileCount;
FSSpec			*inFileArray, *outFileArray;
{
	funcHdl		h;
	OSErr		err;
	Str255		theString;
	FTBlock		theBlock;
	
	if( filters[filterNumber] != 0L) 
	{
		theBlock.inFileCount = inFileCount;
		theBlock.inFileList = inFileArray;
		theBlock.outFileCount = outFileCount;	/* this is a pointer to a long */
		theBlock.outFileList = outFileArray;
		theBlock.callBackProc = (ProcPtr)callBackProc;
		theBlock.message = FILTER_HANDLE_EVENT;
		theBlock.progress = 0;
		theBlock.string0 = (StringPtr)NewPtrClear( sizeof( Str255));
		theBlock.string1 = (StringPtr)NewPtrClear( sizeof( Str255));
		theBlock.string2 = (StringPtr)NewPtrClear( sizeof( Str255));
		theBlock.string3 = (StringPtr)NewPtrClear( sizeof( Str255));
		
		gStopProcessing = FALSE;
		gTheFTBlockPtr = &theBlock;
		h = filters[filterNumber];
		err = (**h)( &theBlock);	/*	h has already been locked down */
		if( err != 0)
		{
			NumToString( (long)err, theString);
			ParamText( theString, "\p", "\p", "\p" );
			StopAlert( 402, 0L);
			gStopProcessing = TRUE;
		}
		DisposePtr((Ptr)theBlock.string0);
		DisposePtr((Ptr)theBlock.string1);
		DisposePtr((Ptr)theBlock.string2);
		DisposePtr((Ptr)theBlock.string3);
		gTheFTBlockPtr = 0L;
	}
	if( gStopProcessing) err = -1;		/* stop all if piping */
	return err;
}

/*************************** callBackProc *****************************/

short		callBackProc( theFTBlockPtr)
FTBlockPtr	theFTBlockPtr;
{	
	OSErr	result;
	
	result = -1;
	
	switch (theFTBlockPtr->message)
	{
		case FILTER_HANDLE_EVENT:
			HandleEvent();
			result = noErr;
			break;

		case DO_ALERT_STRING:
			ParamText( theFTBlockPtr->string0, "\p", "\p", "\p" );
			Alert( 403, 0L);
			result = noErr;
			break;

		case DO_PROGRESS:
			result = HandleProgress( theFTBlockPtr);
			break;

		case GET_USER_STRING:
			result = HandleTextEntryDialog( theFTBlockPtr->string0);
			break;
	}
	if( gStopProcessing == TRUE) theFTBlockPtr->message = -1;	/* stop processing */
	return result;
}

