/* windowUtils.c */

#include		"main.h"
#include		"windowUtils.h"
#include		"filter.h"
#include		"cursors.h"
#include		"events.h"

/****************** Globals ********/

WindowPtr		gFilesWindow, gProgressWindow;
CWindowPtr		gFiltersWindow;
ListHandle		gFileList, gFilterList, gPipeList;
Rect			gFileListRect, gFilterListRect, gPipeListRect;
ControlHandle	gDeleteButton, gDeleteFiltersButton, gPipesButton, gProgressStopButton;

/************************************* WindowInit *********************/

void		WindowInit( void)
{
	Rect	dataBounds;
	Point	cellSize, where;
	Str255	files = "\pFiles", filters = "\pFilters", progress = "\pProgress";
	GrafPtr	oPort;
	
	GetPort( &oPort);
	gProgressWindow = GetNewWindow(402, 0L, 0L);
	SetWTitle( gProgressWindow, progress);
	SetUpProgressButton( gProgressWindow);
	gFiltersWindow = GetNewCWindow(401, 0L, 0L);
	SetWTitle( gFiltersWindow, filters);
	gFilesWindow = GetNewWindow(400, 0L, 0L);
	SetWTitle( gFilesWindow, files);

	SetUpListRect( gFilesWindow, &gFileListRect);
	SetUpListRect( gFiltersWindow, &gFilterListRect);
	SetUpListRect( gFiltersWindow, &gPipeListRect);
	AdjustFilterRects();
	SetRect(&dataBounds, 0, 0, 2, 0);
	SetPt(&cellSize, 530, 18);
	SetPort(gFilesWindow);
	TextFont( 0 );
	TextSize( 12 );
	gFileList = LNew(&gFileListRect, &dataBounds, cellSize, 128, 
					gFilesWindow, true, true, false, true);
	( **gFileList).lClikLoop = (ProcPtr)myLClikLoop;
	SetUpDeleteButton( gFilesWindow);
	SetUpFilterButtons( gFiltersWindow);
	SetPort( gFiltersWindow);
	TextFont( 0);
	TextSize( 12);
	gFilterList = LNew(&gFilterListRect, &dataBounds, cellSize, 130, 
					gFiltersWindow, true, true, false, true);
	( **gFilterList).lClikLoop = (ProcPtr)myLClikLoop;
	gPipeList = LNew(&gPipeListRect, &dataBounds, cellSize, 129, 
					gFiltersWindow, true, true, false, true);
	( **gPipeList).lClikLoop = (ProcPtr)myLClikLoop;
	
	ShowWindow( gFiltersWindow );
	MoveFileWindow();
	SetPort(gFilesWindow);
	ShowWindow( gFilesWindow );
}

/********************* MoveFileWindow **************************************/

void		MoveFileWindow ()
{	
	Point	thePoint;
	
	thePoint.v = gFilesWindow->portRect.top + 20;
	thePoint.h = gFilesWindow->portRect.left + 20;
	LocalToGlobal( &thePoint);
	MoveWindow( gFilesWindow, thePoint.h, thePoint.v, TRUE);
}

/********************* SetUpDeleteButton **************************************/

void		SetUpDeleteButton ( window )
WindowPtr	window;
{	
	Rect	r;
	Str255	Clear = "\pClear";
	
	if( gDeleteButton != 0L)
		DisposeControl( gDeleteButton);
		
	r = window->portRect;
	r.top = r.bottom - 15;
	r.left -= 1;
	r.right = r.left + 70;
	r.bottom += 1;
	
	gDeleteButton = NewControl( window, &r, Clear,
				TRUE, 0, 0, 0, squareButProc, 0L );	

}

/********************* SetUpProgressButton **************************************/

void		SetUpProgressButton ( window )
WindowPtr	window;
{	
	Rect	r;
	Str255	Stop = "\pStop";
	
	if( gProgressStopButton != 0L)
		DisposeControl( gProgressStopButton);
		
	r = window->portRect;
	r.bottom -= 10;
	r.top = r.bottom - 18;
	r.right -= 15;
	r.left = r.right - 55;
	
	gProgressStopButton = NewControl( window, &r, Stop,
				TRUE, 0, 0, 0, pushButProc, 0L );	

}

/********************* SetUpFilterButtons **************************************/

void		SetUpFilterButtons ( window )
WindowPtr	window;
{	
	Rect	r;
	Str255	Clear = "\pClear", Pipe = "\pPipe";
	
	if( gDeleteFiltersButton != 0L)
		DisposeControl( gDeleteFiltersButton);
	if( gPipesButton != 0L)
		DisposeControl( gPipesButton);
		
	r = window->portRect;
	r.right = r.right -14;
	r.top = r.bottom - 15;
	r.left = r.right - 65;
	r.bottom += 1;
	
	gPipesButton = NewControl( window, &r, Pipe,
				TRUE, 0, 0, 0, squareButProc, 0L );	
				
	r.right = r.left +1;
	r.left = r.right - 65;
	
	gDeleteFiltersButton = NewControl( window, &r, Clear,
				TRUE, 0, 0, 0, squareButProc, 0L );	
}

/***************************** AdjustWindowItems **************************/

void		AdjustWindowItems( window)
WindowPtr	window;
{
	if ( window == gFilesWindow )
	{
		SetUpListRect(window, &gFileListRect);
		LSize( gFileListRect.right, gFileListRect.bottom, gFileList );
	}
	else if ( window == gFiltersWindow )
	{
		SetUpListRect( gFiltersWindow, &gFilterListRect);
		SetUpListRect( gFiltersWindow, &gPipeListRect);
		AdjustFilterRects();
		LSize( gFilterListRect.right, gFilterListRect.bottom, gFilterList );
		LSize( gPipeListRect.right - gPipeListRect.left, gPipeListRect.bottom, gPipeList );
	}
	InvalRect( &window->portRect);
}

/**************************** AdjustFilterRects **************************/

void	AdjustFilterRects()
{
	gFilterListRect.right -= (gFiltersWindow->portRect.right) / 2;
	gPipeListRect.left = gFilterListRect.right +16;
}

/********************* SetUpListRect **************************************/

void		SetUpListRect ( window, rect )
WindowPtr	window;
Rect		*rect;
{	
	*rect = window->portRect;
	rect->right -= 15;
 	rect->bottom -= 15;
}

/*************************************** AddLineToList ************************/

void		AddLineToList(list, string)
ListHandle	list;
Str255		string;
{
	Point	cell;
	char	line[63];
	short	i;
	
	SetPt( &cell, 0, (**list).dataBounds.bottom );			/* insert line in list */
	LAddRow( 1, (**list).dataBounds.bottom, list );
	for ( i = 0; i < string[0]; ++i) line[i] = string[i + 1];
	LSetCell( line, string[0], cell, list );
}

/************************* DrawList *************************/

void		DrawList( list)
ListHandle	list;		
{	
	short	i;
	Point	cell;
						
	LDoDraw( TRUE, list );								/* turn filter list drawing on */
	for ( i = 0; i < (**list).dataBounds.bottom; ++i )	/* draw each new line in list */
	{
		SetPt( &cell, 0,  i );
		LDraw( cell, list );
	}
}


/********************************* HandleProgress ****************************/

short			HandleProgress( theBlock)
FTBlockPtr		theBlock;
{
	short		progress;
	WindowPtr	window;
	StringPtr	filterName, fileName;
	GrafPtr		oldPort;
	Rect		r;
	
	progress = theBlock->progress;
	filterName = theBlock->string0;
	fileName = theBlock->string1;
	
	if( progress < 1)
	{
		ShowWindow( gProgressWindow);
		SelectWindow( gProgressWindow);
	}
	else if ( progress > 100)
		HideWindow( gProgressWindow);
	else
	{
		GetPort( &oldPort);
		SetPort( gProgressWindow);
		DrawControls( gProgressWindow);
		TextFont(0);
		TextSize( 12);
		r.top = gProgressWindow->portRect.top;
		r.bottom = r.top + 50;
		r.left = 99;
		r.right = gProgressWindow->portRect.right;
		EraseRect(&r);
		MoveTo( 15, 20);
		DrawString( "\pFilter Name: ");
		MoveTo( 100, 20);
		DrawString( filterName);
		MoveTo( 15, 45);
		DrawString( "\pFile Name: ");
		MoveTo( 100, 45);
		DrawString( fileName);
		r.bottom = gProgressWindow->portRect.bottom - 12;
		r.top = r.bottom - 14;
		r.left = 15;
		r.right = r.left + 202;
		FrameRect( &r);
		r.right = r.left + ( progress * 2) + 2;
		InsetRect( &r, 2, 2);
		PaintRect( &r);
		SetPort( oldPort);
	}
	return 0;
}

/******************************** myLClikLoop ********************************/
/* callback routine from LClick for lists: stuff address into ListRec.lClickLoop */
/* see IM IV 266 */
/* this routine does nothing but loop thru twice and then lie to the List Manager that the mouse is up */
/* it loops thru twice so the List Manager will select the first cell clicked the first loop */
/* it exits on the second loop to make LClick give control of the mouse to the custom routine HandleListDrag */
	
short			gFirstClick = 1;
	
pascal Boolean	myLClikLoop( )
{
	if ( gFirstClick)
	{
		gFirstClick = 0;	/* stay in clikLoop first time thru so List Manager will select first cell clicked on */
		return TRUE;
	}
	else
	{
		gFirstClick = 1;	/* Break out of clickLoop even though mouse still down */
		return FALSE;		/* IM says you should always return TRUE, but we want control */
	}
}

/******************************** HandleFilterListDrag ********************************/
/* Routine called after myLClikLoop has cheekily returned false while the mouse still down */

void			HandleFilterListDrag( list)
ListHandle		list;
{
	Point		mouse;
	Rect		newRect, oldRect;
	Cell		dstCell;
	RgnHandle	theRgn;
	
	if( ( **list).dataBounds.bottom == 0) return;		/* only interested if something in list */
	GetMouse( &mouse);
	oldRect = ( **list).rView;
	if( !PtInRect( mouse, &oldRect)) return;			/* only interested if in rView */
	
	if( !StillDown( )) return;							/* only interested if mouse still down */
	else
	{
		dstCell.h = 0;									
		dstCell.v = 0;
		while( dstCell.v != ( **list).visible.top) dstCell.v += 1;	/* wind thru to first visible cell */
		dstCell.v += mouse.v / ( **list).cellSize.v;				/* add the vertical mouse position */

		if( dstCell.v > ( **list).dataBounds.bottom - 1)	/* keep it within bounds of active cells */
			return;
		else
			LRect( &newRect, dstCell, list);
		newRect.right = ( **list).rView.right;		/* now react to mouse position */
			
		oldRect.right *= 2;
		oldRect.right += 16;
		theRgn = NewRgn();
		RectRgn( theRgn, &newRect);
		DragGrayRgn( theRgn, mouse, &oldRect, &oldRect, 0, 0L);
		DisposeRgn( theRgn);
		GetMouse(&mouse);
		oldRect.left = (oldRect.right / 2) + 16;
		if( PtInRect( mouse, &oldRect))
			HandleFilterLineSelect(dstCell.v);
	}
}

/******************************** HandleListDrag ********************************/
/* Routine called after myLClikLoop has cheekily returned false while the mouse still down */

void			HandleListDrag( list)
ListHandle		list;
{
	Point		mouse;
	Rect		newRect, oldRect;
	Cell		dstCell, srcCell;
	
	if( ( **list).dataBounds.bottom == 0) return;		/* only interested if something in list */
	GetMouse( &mouse);
	oldRect = ( **list).rView;
	if( !PtInRect( mouse, &oldRect)) return;			/* only interested if in rView */
	oldRect.top = oldRect.bottom;						/* so it wont be drawn first time thru loop */
	
	if( !StillDown( )) return;							/* only interested if mouse still down */
	else
	{
		while( WaitMouseUp( ))
		{
			GetMouse( &mouse);							/* Set up newRect. Accomodate scroll state */
			dstCell.h = 0;									
			dstCell.v = 0;
			while( dstCell.v != ( **list).visible.top) dstCell.v += 1;	/* wind thru to first visible cell */
			dstCell.v += mouse.v / ( **list).cellSize.v;				/* add the vertical mouse position */

			if( dstCell.v > ( **list).dataBounds.bottom - 1)	/* keep it within bounds of active cells */
			{
				dstCell.v = ( **list).dataBounds.bottom - 1;
				LRect( &newRect, dstCell, list);
			}
			else
				LRect( &newRect, dstCell, list);
			newRect.right = ( **list).rView.right;		/* now react to mouse position */
			
			if( mouse.v < ( **list).rView.top)			/* scroll down */
			{
				FrameRect( &oldRect);					/* delete old gray rect before scroll */
				LScroll( 0, -1, list);
				oldRect.top = oldRect.bottom;			/* make sure old wont be drawn again */
			}
			else if( mouse.v > ( **list).rView.bottom)	/* scroll up */
			{
				FrameRect( &oldRect);					/* delete old gray rect before scroll */
				LScroll( 0, 1, list);
				oldRect.top = oldRect.bottom;			/* make sure old wont be drawn again */
			}
			else if( newRect.top != oldRect.top)		/* not still in the same cell */
			{
				PenPat( gray);							/* OK, OK, we'll do a gray rect drag */
				PenMode( notPatXor);
				FrameRect( &oldRect);					/* delete old gray rect */
				FrameRect( &newRect);					/* draw new gray rect */
				oldRect = newRect;						/* stuff new into old for next loop */
			}
		}
	}
	FrameRect( &oldRect);								/* delete last gray rect */
	PenNormal( );										/* all drawing done before the shuffle */
	srcCell = LLastClick( list);						/* shuffle the cells */
	
	if( list == gPipeList)
	{
		if( srcCell.v < ( **list).dataBounds.bottom) 
			ShuffleList( srcCell, dstCell, list);
	}
	else if( list == gFileList)
	{
		if( srcCell.v < ( **list).dataBounds.bottom) 
			ShuffleFileList( srcCell, dstCell, list);
	}
}

/******************************* ShuffleList ********************************/
/* move the source row to the position of the destination row */
/* put the label in column 0 for display and hide the FSSpec in column 1 */

void		ShuffleList( srcRow, dstRow, list)
Cell		srcRow, dstRow;
ListHandle	list;
{
	Ptr			dataPtr;
	short		dataLen, pipeInfoSize = sizeof(PipeInfo);
	PipeInfo	myPipeInfo;
	Rect		cellRect;
	
	if ( dstRow.v != srcRow.v)						/* not the same row */
	{
		WatchCursor();
		dataLen = 100;
		dataPtr = NewPtrClear( 100L);
		LGetCell( dataPtr, &dataLen, srcRow, list);	/* get the label */
		srcRow.h = 1;
		LGetCell( &myPipeInfo, &pipeInfoSize, srcRow, list);	/* get the hidden short */
		LDelRow( 1, srcRow.v, list);
		LAddRow( 1, dstRow.v, list);
		LDoDraw( FALSE, list);
		LSetCell( dataPtr, dataLen, dstRow, list);	/* set the label */
		dstRow.h = 1;
		LSetCell( &myPipeInfo, pipeInfoSize, dstRow, list);	/* set the hidden short */
		dstRow.h = 0;
		LDoDraw( TRUE, list);
		LDraw( dstRow, list);
		DisposPtr( dataPtr);
		ArrowCursor();
	}
}

/******************************* ShuffleFileList ********************************/
/* move the source row to the position of the destination row */
/* put the label in column 0 for display and hide the FSSpec in column 1 */

void		ShuffleFileList( srcRow, dstRow, list)
Cell		srcRow, dstRow;
ListHandle	list;
{
	Ptr		dataPtr;
	short	dataLen, specLen = sizeof(FSSpec);
	FSSpec	theSpec;
	
	if ( dstRow.v != srcRow.v)						/* not the same row */
	{
		WatchCursor();
		dataLen = 100;
		dataPtr = NewPtrClear( 100L);
		LGetCell( dataPtr, &dataLen, srcRow, list);	/* get the label */
		srcRow.h = 1;
		LGetCell( &theSpec, &specLen, srcRow, list);	/* get the FSSpec */
		LDelRow( 1, srcRow.v, list);
		LAddRow( 1, dstRow.v, list);
		LSetCell( dataPtr, dataLen, dstRow, list);	/* set the label */
		dstRow.h = 1;
		LSetCell( &theSpec, specLen, dstRow, list);	/* set the FSSpec */
		DisposPtr( dataPtr);
		ArrowCursor();
	}
}


