/* FilterTop 0.15 -- Partial user-interface code, continued... :-) */
/* 3 February 1992   Comments to:  stevej@ais.org                  */

#include <Balloons.h>
#include <EPPC.h>
#include <Folders.h>

#include "defs.h"
#include "Globals.h"
#include "Prototypes.h"

pascal OSErr PlotIconID(Rect *, int, int, int) = {0x303C, 0x0500, 0xABC9};

main()
{
	ToolBoxInit();
/*	Action();              */       /* currently disabled, but functioning perfectly! */
	DefaultPrefsSetup();
	PrefsSetup();    
	WindowSetup();
	RectSetup();
	MenuSetup();
	MiscSetup();   
	DialogSetup(); 
	MainEventLoop();
	Destructor(); 
}

void Destructor(void)
{
	(**winPosition).main = GetWindowPoint(ftWindow);
	(**winPosition).general = GetWindowPoint(generalPrefsDialog);
	(**winPosition).notification = GetWindowPoint(notificationPrefsDialog);
	(**winPosition).creation = GetWindowPoint(fileCreationPrefsDialog);
	(**winPosition).help = GetWindowPoint(hmFTHelpDialog);	

	ChangedResource(winPosition);
	WriteResource(winPosition);
}

Point GetWindowPoint(WindowPtr myWindow)
{
	int		h, v;
	Point	windowPoint;
	GrafPtr oldPort;
	
	GetPort(&oldPort);
	SetPort(myWindow);
	
	h = myWindow->portRect.left;
	v = myWindow->portRect.top;
	windowPoint.h = h;
	windowPoint.v = v;
	LocalToGlobal(&windowPoint);
	SetPort(oldPort);
	
	return windowPoint;
}	

void DefaultPrefsSetup(void)
{
	winPosition = (struct Positions **) NewHandle(sizeof(struct Positions));
	gPrefs = (struct GeneralPrefsBody **) NewHandle(sizeof(struct GeneralPrefsBody));
	fPrefs = (struct FileCreationPrefsBody **) NewHandle(sizeof(struct FileCreationPrefsBody));
	nPrefs = (struct NotificationPrefsBody **) NewHandle(sizeof(struct NotificationPrefsBody));
	
	HLock(winPosition);
	HLock(gPrefs);
	HLock(fPrefs);
	HLock(nPrefs);
	
	(**winPosition).main.h = 100;
	(**winPosition).main.v = 100;
	(**winPosition).general.h = 150;
	(**winPosition).general.v = 150;
	(**winPosition).notification.h = 160;
	(**winPosition).notification.v = 160;
	(**winPosition).creation.h = 170;
	(**winPosition).creation.v = 170;
	(**winPosition).help.h = 180;
	(**winPosition).help.v = 180;

	(**gPrefs).showProgress = ON;
	(**gPrefs).recurse = ON;
	(**gPrefs).deleteOriginal = ON;
	
	(**fPrefs).askNewName = ON;
	(**fPrefs).skip = OFF;
	(**fPrefs).alwaysAsk = OFF;
	(**fPrefs).createUnique = OFF;
	(**fPrefs).overwrite = OFF;
	
	(**nPrefs).nothing1 = ON;
	(**nPrefs).beep1 = OFF;
	(**nPrefs).flash1 = OFF;
	(**nPrefs).alert1 = OFF;
	(**nPrefs).nothing2 = ON;
	(**nPrefs).beep2 = OFF;
	(**nPrefs).flash2 = OFF;
	(**nPrefs).alert2 = OFF;
	
	HUnlock(winPosition);
	HUnlock(gPrefs);
	HUnlock(fPrefs);
	HUnlock(nPrefs);

}

void PrefsSetup(void)
{
	short	foundVRefNum;
	long	foundDirID;
	Boolean	createFolder = true;
	Str255	prefsFileName = "\pFilterTop Prefs";
	OSErr	myErr;
	FInfo	prefsFInfo;
	int		refNum;
	
	myErr = FindFolder(kOnSystemDisk, kPreferencesFolderType, createFolder,
					   &foundVRefNum, &foundDirID);
					   
	if(myErr != noErr)
	{
		done = true;
		return; /* put up dbox to signal error */
	}
	else
	{
			prefsRefNum = HOpenResFile(foundVRefNum, foundDirID, prefsFileName, 0);
	}
	
	if(prefsRefNum == -1)
	{
		HCreateResFile(foundVRefNum, foundDirID, prefsFileName);
		prefsRefNum = HOpenResFile(foundVRefNum, foundDirID, prefsFileName, 0); 
		myErr = HGetFInfo(foundVRefNum, foundDirID, prefsFileName, &prefsFInfo);
		prefsFInfo.fdType = 'Pref';
		prefsFInfo.fdCreator = 'fTOP';
		prefsFInfo.fdFlags = fHasBundle+fOnDesk;
		myErr = HSetFInfo(foundVRefNum, foundDirID, prefsFileName, &prefsFInfo);
		AddResource(winPosition, 'posi', DEFAULT_ID, "\pPositions");
		AddResource(gPrefs, 'genP', DEFAULT_ID+1, "\pGeneral Prefs");
		AddResource(fPrefs, 'filP', DEFAULT_ID+2, "\pFile Creation Prefs");
		AddResource(nPrefs, 'notP', DEFAULT_ID+3, "\pNotification Prefs");
		WriteResource(winPosition);
		WriteResource(gPrefs);
		WriteResource(fPrefs);
		WriteResource(nPrefs);
	}
	else
	{
		/* put up dbox to signal error */
	}	
}

void WindowSetup()
{
	Rect	iconRect;
	long	time;
	int		h, v;
	
	SetRect(&iconRect, 10, 10, 42, 42);

	ftWindow = GetNewCWindow(DEFAULT_ID, 0L, (Ptr) -1L);
	winPosition = (struct Positions **) GetResource('posi', DEFAULT_ID);
	HLock(winPosition);
	h = (**winPosition).main.h;
	v = (**winPosition).main.v; 
	MoveWindow(ftWindow, h, v, true);					 
	HUnlock(winPosition);
	SetPort(ftWindow);
	ShowWindow(ftWindow);
	SelectWindow(ftWindow);
	
	PlotIconID(&iconRect, 0, 0, 129); 
	Delay(60, &time);
}

void RectSetup(void)
{
    boundsRect = (*GetGrayRgn()) -> rgnBBox;
    
 	sizeRect = boundsRect;
	sizeRect.top = 50;
	sizeRect.left = 150;
	sizeRect.bottom -= 40;
}

void MenuSetup(void)
{
	Handle		ftMBar;
	OSErr		menuErr;
	int			index;
	Str255		helpString;
	
	ftMBar = GetNewMBar(DEFAULT_ID);
	SetMenuBar(ftMBar);
	
	appleMenu = GetMHandle(APPLE_MENU);
	AddResMenu(appleMenu, 'DRVR');
	
	prefsSubmenu = GetMenu(PREFS_SUBMENU);
	InsertMenu(prefsSubmenu, hierMenu);
	
	menuErr = HMGetHelpMenuHandle(&helpMenu);
	if(menuErr == noErr)
		if(helpMenu != 0L)
		{
			for(index = 1; index < FT_HM_ITEMS+1; ++index)
			{
				GetIndString(helpString, HELP_BASE_ID, index);
				AppendMenu(helpMenu, helpString);
			}
		}
	DrawMenuBar();
}

void MiscSetup(void)
{

}

pascal void ThickOutline(DialogPtr myDialog, int theItem)
{
	int			itemKind;
	Handle		itemHandle;
	Rect		itemRect;
	PenState	myPenState;
	GrafPtr		oldPort;
	
	GetPort(&oldPort);
	SetPort(myDialog);
	
	GetDItem(myDialog, CANCEL, &itemKind, &itemHandle, &itemRect);
	GetPenState(&myPenState);
	PenSize(3, 3);
	InsetRect(&itemRect, -4, -4);
	FrameRoundRect(&itemRect, 16, 16);
	SetPenState(&myPenState);
	
	SetPort(oldPort);
}

void DialogSetup(void)
{
	int		itemKind;
	Handle	itemHandle;
	Rect	itemRect;
	int		count;

	HLock(winPosition);

	generalPrefsDialog  = GetNewDialog(GENERAL_PREFS_ID, 0L, (Ptr) -1L);
	SetItems(generalPrefsDialog, set, SHOW_PROGRESS_WINDOW, DELETE_ORIGINAL);  
	MoveWindow(generalPrefsDialog, (**winPosition).general.h, (**winPosition).general.v, false);
	
	notificationPrefsDialog  = GetNewDialog(NOTIFICATION_PREFS_ID, 0L, (Ptr) -1L);
	SetItems(notificationPrefsDialog, set, NOP_NOTHING, NOP_NOTHING);
	SetItems(notificationPrefsDialog, set, DONE_NOTHING, DONE_NOTHING);
	MoveWindow(notificationPrefsDialog, (**winPosition).notification.h, (**winPosition).notification.v, false);
	
	fileCreationPrefsDialog  = GetNewDialog(FILE_CREATION_PREFS_ID, 0L, (Ptr) -1L);
	SetItems(fileCreationPrefsDialog, set, ASK_NEW_NAME, ASK_NEW_NAME);
	MoveWindow(fileCreationPrefsDialog, (**winPosition).creation.h, (**winPosition).creation.v, false);
	
	progressDialog = GetNewDialog(PROGRESS_DIALOG_ID, 0L, (Ptr) -1L);
	processPriorityDialog = GetNewDialog(PROCESS_PRIORITY_ID, 0L, (Ptr) -1L);
	
	hmFTHelpDialog = GetNewDialog(FT_HELP_DLG_ID, 0L, (Ptr) -1L);
	MoveWindow(hmFTHelpDialog, (**winPosition).help.h, (**winPosition).help.v, false);

	hmBuildSuperfiltersDialog = GetNewDialog(BUILD_SUPERFILTERS_DLG_ID, 0L, (Ptr) -1L);
	MoveWindow(hmBuildSuperfiltersDialog, (**winPosition).help.h, (**winPosition).help.v, false);

	hmSettingFilterOptionsDialog = GetNewDialog(SETTING_FILTER_OPTS_DLG_ID, 0L, (Ptr) -1L);
	MoveWindow(hmSettingFilterOptionsDialog, (**winPosition).help.h, (**winPosition).help.v, false);

	hmShortcutsDialog = GetNewDialog(SHORTCUTS_DLG_ID, 0L, (Ptr) -1L);
	MoveWindow(hmShortcutsDialog, (**winPosition).help.h, (**winPosition).help.v, false);

	hmTopSoftDialog = GetNewDialog(TOPSOFT_DLG_ID, 0L, (Ptr) -1L);
	MoveWindow(hmTopSoftDialog, (**winPosition).help.h, (**winPosition).help.v, false);
	
	HUnlock(winPosition);
	
	GetDItem(generalPrefsDialog, 6, &itemKind, &itemHandle, &itemRect);
	SetDItem(generalPrefsDialog, 6, itemKind, (Handle) ThickOutline, &itemRect);
	GetDItem(notificationPrefsDialog, 13, &itemKind, &itemHandle, &itemRect);
	SetDItem(notificationPrefsDialog, 13, itemKind, (Handle) ThickOutline, &itemRect);
	GetDItem(fileCreationPrefsDialog, 9, &itemKind, &itemHandle, &itemRect);
	SetDItem(fileCreationPrefsDialog, 9, itemKind, (Handle) ThickOutline, &itemRect);
	GetDItem(processPriorityDialog, 7, &itemKind, &itemHandle, &itemRect);
	SetDItem(processPriorityDialog, 7, itemKind, (Handle) ThickOutline, &itemRect);
}

void SetItems(DialogPtr thisDialog, int value, int startItem, int endItem)
{
	int		itemKind, count;
	Handle	itemHandle;
	Rect	itemRect;
	
	for(count = startItem; count < endItem+1; count++)
	{
		GetDItem(thisDialog, count, &itemKind, &itemHandle, &itemRect);
		SetCtlValue(itemHandle, value);
	}
}

void ToolBoxInit(void)
{
	InitGraf(&thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(0L);
	InitCursor();
	MaxApplZone();
}

void DoDialogEvent(void)
{
	DialogPtr	hitDialog;
	int			itemHit;
	Boolean		validHit;
	char		keyPressed;
	
	if(myEvent.what == keyDown)
	{
		keyPressed = (myEvent.message) & charCodeMask;
		
		if((keyPressed == ENTER) || (keyPressed == CR))
		{
			SaveChoices(FrontWindow());
			HideWindow(FrontWindow());
		}	
		return;
	}
	
	validHit = DialogSelect(&myEvent, &hitDialog, &itemHit);
	if(validHit == true)
		HandleModelessHit(hitDialog, itemHit);
}

void SaveChoices(WindowPtr myWindow)
{
	Rect	winRect;
	Point	topLeftPoint;
	
	/* save dbox choices, then... window positions */
	
/*	winRect = myWindow->portRect;
	topLeftPoint.h = portRect.left;
	topLeftPoint.v = portRect.top;
	LocalToGlobal(&topLeftPoint);
	
	winPosition.main = topLeftPoint; */
}

void MainEventLoop(void)
{
	int			eventType;
	Boolean		gotEvent, isDialogEvent;
	Boolean		validHit;
	DialogPtr	hitDialog;
	int			itemHit;

	while(done==false)
	{
		gotEvent = WaitNextEvent(everyEvent, &myEvent, sleepTicks, currentRegion);
 		isDialogEvent = IsDialogEvent(&myEvent);
		
		if(isDialogEvent == true)
		{				
			DoDialogEvent(); 
		}
		else      
		
		if(gotEvent == true)
	 	{
	 		 eventType = myEvent.what;
	 	
	 		switch(eventType)
	 		{
	 			case	keyDown:
	 			case	autoKey:
	 			
	 					HandleKeyDown(myEvent.message, myEvent.modifiers);
	 					break;
	 					
	 			case	mouseDown:
	 			
	 					HandleClick(&myEvent);
	 					break;
	 					
	 			case	updateEvt:
	 			
	 					HandleUpdate((WindowPtr) myEvent.message);
	 					break;
	 					
	 			case	activateEvt:
	 			
	 					HandleActivate((WindowPtr) myEvent.message);
	 					break;
	 					
	 			case	osEvt:
	 			
	 					HandleOSEvent(&myEvent);
	 					break;
	 					
	 			case	kHighLevelEvent:
	 			
	 					HandleHighLevelEvent();
	 					break;
	 		

	 			case	nullEvent:
	 			
	 					HandleNull();
	 					break;
	 					
	 		}
	 	}
	}
}

void HandleModelessHit(DialogPtr thisDialog, int itemHit)
{
	int		itemKind;
	Handle	itemHandle;
	Rect	itemRect;

	if((itemHit == SAVE) || (itemHit == CANCEL))
	{
		HideWindow(thisDialog);
		return;
	}

	if(thisDialog == generalPrefsDialog)
	{	
		GetDItem(thisDialog, itemHit, &itemKind, &itemHandle, &itemRect);
		SetCtlValue(itemHandle, !GetCtlValue(itemHandle));
	} 
	else if(thisDialog == notificationPrefsDialog)
	{
		switch(itemHit)
		{
			case	NOP_NOTHING:
			case	NOP_BEEP:
			case	NOP_ICON_FLASH:
			case	NOP_ALERT:

					GetDItem(thisDialog, notificationNotChoice, &itemKind, &itemHandle, &itemRect);
					SetCtlValue(itemHandle, clear);
				
					GetDItem(thisDialog, itemHit, &itemKind, &itemHandle, &itemRect);
					SetCtlValue(itemHandle, !GetCtlValue(itemHandle));
					notificationNotChoice = itemHit;
					
					break;
					
			case	DONE_NOTHING:
			case	DONE_BEEP:
			case	DONE_ICON_FLASH:
			case	DONE_ALERT:

					GetDItem(thisDialog, notificationDoneChoice, &itemKind, &itemHandle, &itemRect);
					SetCtlValue(itemHandle, clear);
				
					GetDItem(thisDialog, itemHit, &itemKind, &itemHandle, &itemRect);
					SetCtlValue(itemHandle, !GetCtlValue(itemHandle));
					notificationDoneChoice = itemHit;
					
					break;
		}						
	}
	else if(thisDialog == fileCreationPrefsDialog)
	{
		GetDItem(thisDialog, fileCreationChoice, &itemKind, &itemHandle, &itemRect);
		SetCtlValue(itemHandle, clear);
		
		GetDItem(thisDialog, itemHit, &itemKind, &itemHandle, &itemRect);
		SetCtlValue(itemHandle, set);
		fileCreationChoice = itemHit;
	}
		 
}

void HandleKeyDown(long key, int modifiers)
{
	if(modifiers & cmdKey)
		HandleMenu(MenuKey(key));
}	

void HandleMenu(long menuCode)
{
	int		thisMenu, thisItem;
	
	thisMenu = HiWord(menuCode);
	thisItem = LoWord(menuCode);
	
	switch(thisMenu)
	{
		case	APPLE_MENU:
		
				HandleApple(thisItem);
				break;
				
		case	FILE_MENU:
		
				HandleFile(thisItem);
				break;
			
		case	EDIT_MENU:
		
				HandleEdit(thisItem);
				break;
				
		case	FILTERS_MENU:
		
				HandleFilters(thisItem);
				break;
				
		case	PREFS_SUBMENU:
		
				HandlePrefs(thisItem);
				break;
				
		case	WINDOW_MENU:
		
				HandleWindow(thisItem);
				break;
				
		case	kHMHelpMenuID:
		
				HandleHelp(thisItem);
				break;
	}
	HiliteMenu(0);
}
 	
void HandleApple(int thisItem)
{
	Str255		daName;
	
	switch(thisItem)
	{
		case	ABOUT_FILTERTOP:
		
			/*	Play(DEFAULT_ID); */
				NoteAlert(DEFAULT_ID, 0L);
				break;
				
		case	SET_PROCESS_PRIORITY:
		
				ShowDialog(processPriorityDialog);
				break;
				
		default:
				
				GetItem(appleMenu, thisItem, daName);
				OpenDeskAcc(daName);
				break;
	}
	HiliteMenu(0);
}

void SetPriority(void)
{

}

void HandleFile(int thisItem)
{
	switch(thisItem)
	{
		case	NEW_SUPERFILTER:
		
				HandleNewSuperfilter();
				break;
				
		case	OPEN:
		
				HandleOpen();
				break;
				
		case	CLOSE:
		
				HandleClose();
				break;
				
		case	SAVE_ITEM:
		case	SAVE_AS:
		
				HandleSave();
				break;
				
		case	REVERT:
		
				HandleRevert();
				break;
				
		case	TRANSFORM_FILE:
		
				HandleTransform();
				break;
				
		case	GET_INFO:
		
				HandleGetInfo();
				break;
				
		case	PUBLISH:
		
				HandlePublish();
				break;
				
		case	SUBSCRIBE:
		
				HandleSubscribe();
				break;
				
		case	PAGE_SETUP:
		
				HandlePageSetup();
				break;
				
		case	PRINT:
		
				HandlePrint();
				break;
				
		case	QUIT:
		
				HandleQuit();
				break;
	}	
	HiliteMenu(0);	
}

void HandleNewSuperfilter(void)
{

}

void HandleOpen(void)
{

}

void HandleClose(void)
{

}

void HandleSave(void)
{

}

void HandleRevert(void)
{

}

void HandleTransform(void)
{

}

void HandleGetInfo(void)
{

}

void HandlePublish(void)
{

}

void HandleSubscribe(void)
{

}

void HandlePageSetup(void)
{

}

void HandlePrint(void)
{

}

void HandleQuit(void)
{
	HiliteMenu(FILE_MENU);
	done = true;
	HiliteMenu(0);
}

void HandleEdit(int thisItem)
{
	switch(thisItem)
	{
	
		case	UNDO:
		
				HandleUndo();
				break;
				
		case	CUT:
		
				HandleCut();
				break;
				
		case	COPY:
		
				HandleCopy();
				break;
				
		case	PASTE:
		
				HandlePaste();
				break;
				
		case	CLEAR:
		
				HandleClear();
				break;
				
		case	SELECT_ALL:
		
				HandleSelectAll();
				break;
				
		case	ATTACH_FOLDER:
		
				HandleAttachFolder();
				break;
	}
	HiliteMenu(0);
}
 
void HandleUndo(void)
{

}

void HandleCut(void)
{

}

void HandleCopy(void)
{

}

void HandlePaste(void)
{

}

void HandleClear(void)
{

}

void HandleSelectAll(void)
{

}

void HandleAttachFolder(void)
{

}
 
void HandleFilters(int thisItem)
{
	switch(thisItem)
	{
		case	SET_DEFAULT_OPTIONS:
		
				HandleSetDefaultOptions();
				break;
				
		case	REDIRECT_OUTPUT:
		
				HandleRedirectOutput();
				break;
				
		case	INTELLIGENCE:
		
				HandleIntelligence();
				break;
	}
	HiliteMenu(0);
}

void HandleSetDefaultOptions(void)
{

}

void HandleRedirectOutput(void)
{

}

void HandleIntelligence(void)
{

}

void HandlePrefs(int thisItem)
{
	switch(thisItem)
	{
		case	GENERAL:
		
				ShowDialog(generalPrefsDialog);
				break;
				
		case	NOTIFICATION:
		
				ShowDialog(notificationPrefsDialog);
				break;
				
		case	FILE_CREATION:
		
				ShowDialog(fileCreationPrefsDialog);
				break;
	}
	HiliteMenu(0);
}

void ShowDialog(GrafPtr myDialog)
{
	ShowWindow(myDialog);
	SelectWindow(myDialog);
}
	
void HandleWindow(int thisItem)
{
	/* just select the window chosen */
}

void HandleHelp(int thisItem)
{
	switch(thisItem)
	{
		case	FILTERTOP_HELP:
		
				ShowDialog(hmFTHelpDialog);
				break;
				
		case	BUILDING_SUPERFILTERS:
		
				ShowDialog(hmBuildSuperfiltersDialog);
				break;
				
		case	SETTING_FILTER_OPTIONS:
		
				ShowDialog(hmSettingFilterOptionsDialog);
				break;
				
		case	SHORTCUTS:
		
				ShowDialog(hmShortcutsDialog);
				break;
				
		case	ABOUT_TOPSOFT:
		
				ShowDialog(hmTopSoftDialog);
				break;
	}
}
 
void HandleClick(EventRecord *myEvent)
{
	int			where;
	long		menuCode;
	WindowPtr	clickedWindow;
	
	where = FindWindow(myEvent->where, &clickedWindow);
	
	switch(where)
	{
		case	inMenuBar:
		
				menuCode = MenuSelect(myEvent->where);
				HandleMenu(menuCode);
				break;
				
		case	inSysWindow:
		
				SystemClick(myEvent, clickedWindow);
				break;
				
		case	inContent:
		
				SelectWindow(clickedWindow);
				break;
				
		case	inDrag:
		
				SelectWindow(clickedWindow);    
				DragWindow(clickedWindow, myEvent->where, &boundsRect);
				break;
				
		case	inGoAway:
		
				HandleGoAway(clickedWindow, where, myEvent->where);
				break;
				
		case	inGrow:
		
				HandleGrow(clickedWindow, myEvent->where);
				break;
				
		case	inZoomIn:
		case	inZoomOut:
		
				HandleZoom(clickedWindow, where, myEvent->where);
				break;
	}
}

void HandleZoom(WindowPtr clickedWindow, int partCode, Point downHere)
{
	if(TrackBox(clickedWindow, downHere, partCode))
	{
		ZoomWindow(clickedWindow, partCode, true);
	}
}

void HandleGoAway(WindowPtr clickedWindow, int partCode, Point downHere)
{
	Boolean		clickedBox;
	
	clickedBox = TrackBox(clickedWindow, downHere, partCode);
		if((clickedBox == true) && (clickedWindow == ftWindow))
			done = true;
	else
		HideWindow(clickedWindow);
}

void HandleGrow(WindowPtr thisWindow, Point clickedHere)
{
	long	location;
	int		h, v;
	
	location = GrowWindow(thisWindow, clickedHere, &sizeRect);
	h = LoWord(location);
	v = HiWord(location);
	SizeWindow(thisWindow, h, v, false);
	InvalRect(&thisWindow->portRect);
}

void HandleUpdate(WindowPtr updateWindow)
{
	InvalRect(&updateWindow->portRect);

	BeginUpdate(updateWindow);
	EraseRect(&updateWindow->portRect);
	if(updateWindow == ftWindow)
		DrawGrowIcon(updateWindow);
	EndUpdate(updateWindow);
}

void HandleActivate(WindowPtr activateWindow)
{
	if(activateWindow == ftWindow)
		DrawGrowIcon(activateWindow);
}

void HandleOSEvent(EventRecord *myEvent)
{

}

void HandleHighLevelEvent(void)
{

}

void HandleNull(void)
{

}	 	
	 	
 	
 	
 	
 	
 	
 	
 	
 	
 	
 	
 	