#include <Traps.h>
#include <GestaltEqu.h>

#include "Prototypes.h"

extern	Boolean	done;
const	int		START_ID = 128;
const	int		STOP_INDEX = 1;
const	int		GESTALT_ERROR_ID = 129;


/************************ Cookbook functions from IM 6, 3-8 **********************/

int	NumToolboxTraps(void)
{
	if(NGetTrapAddress(_InitGraf, ToolTrap) == NGetTrapAddress(0xAA6E, ToolTrap))
		return 0x0200;
	else
		return 0x0400;
}

TrapType GetTrapType(int theTrap)
{
	const trapMask = 0x0800;
	
	if(BitAnd(theTrap, trapMask) > 0 )
		return ToolTrap;
	else
		return OSTrap;
}

Boolean TrapAvailable(int theTrap)
{
	TrapType 	tType;
	Boolean		result;
	
	tType = GetTrapType(theTrap);
	if(tType == ToolTrap)
	{
		theTrap = BitAnd(theTrap, 0x07FF);
		if(theTrap >= NumToolboxTraps())
			theTrap = _Unimplemented;
	}
	result = (NGetTrapAddress(theTrap, tType) != NGetTrapAddress(_Unimplemented, ToolTrap));
	return result;
}

/**************** Determining whether Gestalt is Available, IM 6 3-32 *******************/

Boolean GestaltAvailable(void)
{
	const	_Gestalt = 0xA1AD;
	Boolean	result;
	
	result = TrapAvailable(_Gestalt);
	return result;
}

/************* Based on Cookbook source from IM 6, 3-43 ********************/

Boolean AddNewSelector(void)
{
	const	OSType	mySelector = 'fTOP';
	const	OSType	gstFuncRsrcType = 'GDEF';
	const	int		gstFuncRsrcID = 128;
	const	int		RSRC_PROBLEM = 1;
	const	int		NO_ADD = 2;
	const	int		TRY_REPLACE = 3;
	const	int		NO_REPLACE = 4;
	const	int		INSTALLED = 5;
	
	OSErr	gestaltErr;
	Handle	gstFuncHandle;
	ProcPtr	oldGestaltFunc;
	long	result;
	
	gstFuncHandle = GetResource(gstFuncRsrcType, gstFuncRsrcID);
	if(gstFuncHandle == nil)
	{
		Write(RSRC_PROBLEM, 0, 0, 0);
		done = true;
		return true;
	}
	DetachResource(gstFuncHandle);
	gestaltErr = NewGestalt(mySelector, (ProcPtr) (*gstFuncHandle));
	if(gestaltErr != noErr)
	{
		Write(NO_ADD, TRY_REPLACE, 0, 0);
		gestaltErr = ReplaceGestalt(mySelector, (ProcPtr) (*gstFuncHandle), oldGestaltFunc);
		if(gestaltErr != noErr)
		{
			Write(NO_REPLACE, 0, 0, 0);
			done = true;
			return true;
		}
	}
	Write(INSTALLED, 0, 0, 0);
	
	return false;
}

void Action(void)
{
	Boolean	hasGestalt;
	Str255	myString;
	Boolean	problem;
	
	hasGestalt = GestaltAvailable();
	
	if(hasGestalt)
	{
		problem = AddNewSelector();
		if(!problem)
			TestNewSelector();		
	}
	else 
	{
		GetIndString(myString, START_ID, STOP_INDEX);
		ParamText(myString, "\p", "\p", "\p");
		StopAlert(START_ID, 0L);
		done = true;
	}
}

void Write(int index1, int index2, int index3, int index4)
{
	Str255	param1, param2, param3, param4;
	
	GetIndString(param1, GESTALT_ERROR_ID, index1);
	GetIndString(param2, GESTALT_ERROR_ID, index2);
	GetIndString(param3, GESTALT_ERROR_ID, index3);
	GetIndString(param4, GESTALT_ERROR_ID, index4);
	
	ParamText(param1, param2, param3, param4);
	NoteAlert(START_ID, 0L);		     		 
}

void TestNewSelector(void)
{
	OSErr	myGestaltErr;
	long	result;
	long	i;
	
	myGestaltErr = Gestalt('fTOP', &result);
	for(i=0;i<result;++i)
		SysBeep(10);
}
