/*
========================================================================================

	LDEF+sicn
	
	Douglas Yarrington
	yarri@ee.eng.ohio-state.edu
	
	I received this posting from over the think-c mail group. Mark Fleming updated
	it from Josh Pritikin's code so that it would behave correctly with color
	hilites.
	
	I added code to allow for the user to specify their own sicn resource handle
	in the list's userHandle, and I allowed the text in the list to be displayed
	using the current grafPort's text style.
	
	From: FLEMINGM@qucdn.queensu.ca ("R. Mark Fleming")
	
	Here's an example of LDEF for SICN with Text.   I just updated it to
	also work with the current Hilite Color.

	From: 6500stom@ucsbuxa.ucsb.edu
	
	sicn ldef.c  v1.0
	Copyright 1989 Josh Pritikin
	All rights reserved.

	This LDEF may be used freely in any programming project, no strings attached.

	History

	12.19.89	v1.0 Release
	12.19.89	v1.02 Release - (sorry 'bout not making this the first release)
				
	1.24.92		DNY	Added user specifiable sicn resource handle.
	1.28.92		DNY	Ready for release with specifiable sicn resource handles.
					

========================================================================================
*/

/* #define testing */

#ifdef testing
pascal void ldef128(int message, Boolean select, Rect *bounds, Cell cell,
	int DataOff, int DataLen, ListHandle lhandle)
#else
pascal void main(int message, Boolean select, Rect *bounds, Cell cell,
	int DataOff, int DataLen, ListHandle lhandle)
#endif
{
	Rect	savedOrig;
	GrafPtr	aPort, saveport;
	int		savedMode, savedPenVis;
	PenState	ps;
    char *p;   /* Pointer used to clear Color QD flag RMF */

	if (message == lDrawMsg || message == lHiliteMsg) {
		GetPort(&saveport);
		aPort = (**lhandle).port;
		SetPort(aPort);

		savedOrig = aPort->portRect;
		SetOrigin(0,0);
		savedPenVis = aPort->pnVis;
		aPort->pnVis = 0;
		GetPenState(&ps);

		savedMode = aPort->txMode;
		// DNY: Removed the following to allow display font to be
		//		the same as the list's grafPort 
		// TextFont(systemFont);
		// TextSize(0);
		// TextFace(0);
		TextMode(srcOr);
	}

	switch (message) {
	case lInitMsg:
		{
		int		id;
		long	type;
		Str255	name;

		// DNY
		// Check for null userHandle, if null then add handle to sicn's
		// resouce of same id as LDEF, else user has supplied handle
		if ((**lhandle).userHandle == 0L)		// Null, add handle to sicn's
		{
			GetResInfo((**lhandle).listDefProc, &id, &type, &name);
			(**lhandle).userHandle = GetResource('SICN', id);
			HPurge((**lhandle).userHandle);
		}

		break;
		}

	case lDrawMsg:
		{
		Handle	sicns;
		int		index;
		Boolean	hilited;
		int		strLen;		// Added 1.27.92 DNY

		EraseRect(bounds);
		MoveTo(bounds->left + 22, bounds->bottom - 4);
		strLen = (int)(*(**lhandle).cells + DataOff)[2];
		DrawText(*(**lhandle).cells, DataOff+3, strLen);

		// DNY: Changed index to an int
		index = (int)*(*(Handle)(**lhandle).cells + DataOff);
		index <<= 8;
		index = (int)*(*(Handle)(**lhandle).cells + DataOff + 1);
		
		if (index >= 0x80) {
			index = index & 0x7f;
			hilited = false;
		} else
			hilited = true;

		sicns = (**lhandle).userHandle;
		if (sicns) {
			LoadResource(sicns);	// what about this?
									// assume sicn's loaded and let the app
									// handle loading/purging of resource
			if (*sicns) {
				BitMap	bmap;
				Rect	r;

				HNoPurge(sicns);
				HLock(sicns);

				bmap.baseAddr = *sicns+(index * 32);
				SetRect(&bmap.bounds, 0, 0, 16, 16);
				bmap.rowBytes = 2;

				r = bmap.bounds;
				OffsetRect(&r, bounds->left+3, bounds->top);

				CopyBits(&bmap, &aPort->portBits, &bmap.bounds, &r, srcOr,
					nil);

				HUnlock(sicns);
				HPurge(sicns);
			}
		}

		if (!hilited) {
			long	gray[2];

			PenMode(patBic);
			gray[0] = gray[1] = 0xaa55aa55;		/* 50% gray */
			PenPat(gray);
			PaintRect(bounds);
		}

		if (!select)
			break;
		}

	case lHiliteMsg:
		{
          p = (char *) 0x938;   /* Flag Invert Color */
          *p = *p & 0x7f;  /* HiLiteMode for Color QD - RMF  */
		if (*(*(Handle)(**lhandle).cells + DataOff) < 0x80)
			InvertRect(bounds);
		break;
		}

	case lCloseMsg:
		{
		if ((**lhandle).userHandle)
			ReleaseResource((**lhandle).userHandle); // This should also be changed
		break;
		}
	}

	if (message == lDrawMsg || message == lHiliteMsg) {
		SetPenState(&ps);
		aPort->pnVis = savedPenVis;
		aPort->portRect = savedOrig;
		TextMode(savedMode);
		SetPort(saveport);
	}
}


