#include "sky.h"

extern struct suntab
{
	float f[2];
	char c[3];
} suntab[];
/*
 *	WARNING: Requires sign extension of characters.
 */

sun()
{
	double mmerc, mven, merth, mmars, mjup, msat;
	double dmoon, mmoon, gmoon;
	double pturbb, pturbl, pturbr, lograd;
	double planp[7];
	struct suntab *pp;
	pp = &suntab[0];

/*
 *	The arugments and coefficients are taken from
 *	Simon Newcomb, "Tables of the Motion of the Earth upon
 *	its Axis and About the Sun", A.P.A.E. VI (1895)
 */

	object = "sun         ";
	ecc = .01675104 - 4.180e-5 * capt - 1.26e-7*capt2;
	incl = 0.;
	node = 0.;
	argp = 281.220833 + .0000470684*eday + .000453*capt2
		 + .000003*capt3
		 + .000200;
	mrad = 1.0;
	anom = 358.475845 + .9856002670*eday - .000150*capt2
		 - .000003*capt3
		 - .000200;
	motion = .9856473354;

	dmoon = 350.737681+12.1907491914*eday-.001436*capt2;
	gmoon = 11.250889 + 13.2293504490*eday - .003212*capt2;
	mmoon = 296.104608 + 13.0649924465*eday + 9.192e-3*capt2;
	mmerc = 102.19 + 4.092329959*eday;
	mven  = 212.448 + 1.602121635*eday;
	merth = 358.476 + 0.985600267*eday;
	mmars = 319.590 + .524024095*eday;
	mjup  = 225.269 + .083082362*eday;
	msat  = 175.593 + .033450794*eday;
/*	muran = 72.248 + .011722663*eday;*/

	dmoon = fmod(dmoon, 360.)*radian;
	gmoon = fmod(gmoon, 360.)*radian;
	mmoon = fmod(mmoon, 360.)*radian;
	mmerc *= radian;
	mven  *= radian;
	merth *= radian;
	mmars *= radian;
	mjup *= radian;
	msat  *= radian;


/*
	long period terms in the mean anomaly - the arguments
	of the terms are:
		4*mars - 7*earth + 3*venus
		3*jup - 8*mars + 4*earth
		13*earth - 8*venus
		8*earth - 15*mars
*/

	anom = anom
	  - 0.265/3600.*cos(4.*mmars - 7.*merth + 3.*mven)
	  - 0.021/3600.*sin(4.*mmars - 7.*merth + 3.*mven)
	  + 6.40 /3600.*sin((231.19 + 20.20*capt)*radian)
	  + 1.870/3600.*sin((57.24+150.27*capt)*radian);

	incl *= radian;
	node *= radian;
	argp *= radian;
	anom = fmod(anom, 360.)*radian;

/*
 *	computation of elliptic orbit
 */

	lambda = anom + argp;

	pturbl = (6910.057 - 17.240*capt - 0.052*capt2)*sin(anom)
		 + (72.338 - 0.361*capt) * sin(2.*anom)
		 + (1.054 - 0.001*capt) * sin(3.*anom)
		 + 0.018 * sin(4.*anom);

	lambda += pturbl*radsec;

	beta = 0.;

	lograd = (30.57e-6 - 0.15e-6*capt)
		 - (7274.12e-6 - 18.14e-6*capt - 0.05e-6*capt2)*cos(anom)
		 - (91.38e-6 - 0.46e-6*capt) * cos(2.*anom)
		 - (1.45e-6 - 0.01e-6*capt) * cos(3.*anom)
		 - 0.02e-6 * cos(4.*anom);

/*
 *	Computation of perturbations to elliptic orbit.
 */

	planp[1] = mmerc;
	planp[2] = mven;
	planp[3] = merth;
	planp[4] = mmars;
	planp[5] = mjup;
	planp[6] = msat;

	pturbl = 0.;
	for(;;){
		if(pp->f[0]==0.){
			pp++;
			break;
		}
		pturbl += pp->f[0]*cos(pp->f[1] + pp->c[0]*merth + pp->c[1]*planp[pp->c[2]]);
		pp++;
	}

	pturbl = pturbl
	  + 6.454*sin(dmoon)
	  + 0.013*sin(3.*dmoon)
	  + 0.177*sin(dmoon + mmoon)
	  - 0.424*sin(dmoon - mmoon)
	  + 0.039*sin(3.*dmoon - mmoon)
	  - 0.064*sin(dmoon + merth)
	  + 0.172*sin(dmoon - merth)
	  - 0.013*sin(dmoon-mmoon-merth)
	  - 0.013*sin(2.*gmoon-2.*dmoon);

	pturbl *= radsec;


	pturbb = 0.;
	for(;;){
		if(pp->f[0]==0.){
			pp++;
			break;
		}
		pturbb += pp->f[0]*cos(pp->f[1] + pp->c[0]*merth + pp->c[1]*planp[pp->c[2]]);
		pp++;
	}


	pturbb = pturbb
	  + 0.576*sin(gmoon)
	  + 0.016*sin(gmoon + mmoon)
	  - 0.047*sin(gmoon - mmoon)
	  + 0.021*sin(2.*dmoon - gmoon)
	  + 0.005*sin(2.*dmoon-gmoon-mmoon)
	  + 0.005*sin(gmoon+merth)
	  + 0.005*sin(gmoon-merth);

	pturbb *= radsec;


	pturbr = 0.;
	for(;;){
		if(pp->f[0]==0.){
			pp++;
			break;
		}
		pturbr += pp->f[0]*cos(pp->f[1] + pp->c[0]*merth + pp->c[1]*planp[pp->c[2]]);
		pp++;
	}


	pturbr = pturbr
	  +13.360e-6*cos(dmoon)
	  + 0.030e-6*cos(3.*dmoon)
	  + 0.370e-6*cos(dmoon + mmoon)
	  - 1.330e-6*cos(dmoon - mmoon)
	  + 0.080e-6*cos(3.*dmoon - mmoon)
	  - 0.140e-6*cos(dmoon + merth)
	  + 0.360e-6*cos(dmoon - merth)
	  - 0.030e-6*cos(dmoon-mmoon-merth)
	  + 0.030e-6*cos(2.*gmoon-2.*dmoon);

	lambda += pturbl;
	if(lambda > 2.*pi)
		lambda -= 2.*pi;

	beta += pturbb;

	lograd = (lograd+pturbr) * 2.30258509;
	rad = 1. + lograd * (1. + lograd * (.5 + lograd/6.));

	motion *= radian*mrad*mrad/(rad*rad);

	semi = 961.182;
	if(flags & OCCULT)
		semi = 959.63;
	mag = -26.5;
}
